#include "system.h"

unsigned char COMET_readReg(int baseAddress, unsigned char address) {
	unsigned char value;
//	value = *((unsigned char *)baseAddress+(address<<1));
	value = *((unsigned char *)baseAddress+(address<<2));
	return value;
}

void COMET_writeReg(int baseAddress, unsigned char address, unsigned char value) {
	// A8:0 of the COMET chips are connected to A9:1 of the Blackfin
//	unsigned char *externalAddress = (unsigned char *)(baseAddress+(address<<1));
	unsigned char *externalAddress = (unsigned char *)(baseAddress+(address<<2));
	*externalAddress = value;
}

void COMET_writeReg_verify(int baseAddress, unsigned char address, unsigned char value) {
	// A8:0 of the COMET chips are connected to A9:1 of the Blackfin
//	unsigned char *externalAddress = (unsigned char *)(baseAddress+(address<<1));
	unsigned char *externalAddress = (unsigned char *)(baseAddress+(address<<2));
	unsigned char temp;
	*externalAddress = value;
#ifdef DEBUG_COMET			// check written value

	ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();ssync();
	if ((temp=COMET_readReg(baseAddress, address)) != value)
			printf("Warning: Address %x %x:  written: %x   read: %x \n",baseAddress, address, value, temp );
#endif
}

int COMET_readRAM(int baseAddress, unsigned char address) {
	int value = 0;
	int i;
	
	COMET_writeReg(baseAddress, 0x0FC, address);
	
	// issue a read command
	COMET_writeReg(baseAddress, 0x0FD, 0x80);
		
// insert a 3 line rate cycle delay here  (for now just a delay loop that is "long enough")
	i=0;
	while(i < 10000) 
			{ asm("nop;");
			  i++;
			}
	
	value |= (COMET_readReg(baseAddress, 0x0D8))<<24;	
	value |= (COMET_readReg(baseAddress, 0x0D9))<<16;	
	value |= (COMET_readReg(baseAddress, 0x0DA))<<8;	
	value |= (COMET_readReg(baseAddress, 0x0DB))<<0;	
	
	return value;
}

void COMET_writeRAM(int baseAddress, unsigned char address, int value) {
	int i;
	
	COMET_writeReg(baseAddress, 0x0D8, (value&0xFF000000)>>24);	
	COMET_writeReg(baseAddress, 0x0D9, (value&0x00FF0000)>>16);	
	COMET_writeReg(baseAddress, 0x0DA, (value&0x0000FF00)>>8);	
	COMET_writeReg(baseAddress, 0x0DB, (value&0x000000FF)>>0);

	COMET_writeReg(baseAddress, 0x0FC, address);
	
	// issue a write command
	COMET_writeReg(baseAddress, 0x0FD, 0x00);
	
// insert a 3 line rate cycle delay here  (for now just a delay loop that is "long enough")
	i=0;
	while(i < 10000) 
			{ asm("nop;");
			  i++;
			}
}

void COMET_reset(int baseAddress1, int baseAddress2) {
	int temp;
	
	COMET_writeReg(baseAddress1, 0x0E, 0x01);
	COMET_writeReg(baseAddress2, 0x0E, 0x01);

	for (temp=1; temp<10000000; temp++) ssync();		// hold reset for a while
	
	COMET_writeReg(baseAddress1, 0x0E, 0x00);
	COMET_writeReg(baseAddress2, 0x0E, 0x00);

	for (temp=1; temp<10000000; temp++) ssync();		// wait for a while before configuring Comet
}

void COMET_clearPulseTemplate(int baseAddress) {
	int i, j;
	unsigned char indirectAddress;
	for (i = 0; i <= 4; i++) {
		for (j = 0; j <= 23; j++) {
			indirectAddress = (j<<3)|i;
			COMET_writeReg(baseAddress, 0xF2, indirectAddress);
		COMET_writeReg(baseAddress, 0xF3, 0x00);
		}
	}
}

void COMET_initPulseTemplate(int baseAddress, unsigned char transmitWaveform[24][5]) {
	int i, j;
	unsigned int indirectAddress;
	for (i = 0; i <= 4; i++) {
		for (j = 0; j <= 23; j++) {
			indirectAddress = (j<<3)|i;
			COMET_writeReg(baseAddress, 0xF2, indirectAddress);
			COMET_writeReg(baseAddress, 0xF3, transmitWaveform[j][i]);
		}
	}	
}

void COMET_stabilizeFuses(int baseAddress) {
	unsigned char value;
	/* write the value 0x01 to register XLPG Analog Test Positive
	Control(0xF4) twice and read the result and _AND_ it with 0xFE and
	rewrite it to 0xF4*/
	COMET_writeReg(baseAddress, 0xF4, 0x01);
	COMET_writeReg(baseAddress, 0xF4, 0x01);
	value = (COMET_readReg(baseAddress, 0xF4) & 0xFE);
	COMET_writeReg(baseAddress, 0xF4, value);
	
	/* same as above except for register XLPG Analog Test Negative
	Control(0xF5)*/
	COMET_writeReg(baseAddress, 0xF5, 0x01);
	COMET_writeReg(baseAddress, 0xF5, 0x01);
	value = (COMET_readReg(baseAddress, 0xF5) & 0xFE);
	COMET_writeReg(baseAddress, 0xF5, value);
	/* set up the Fuse Data Select */
	COMET_writeReg(baseAddress, 0xF6, 0x01);
}

void COMET_configurePulseTemplate(int baseAddress, unsigned char transmitWaveform[24][5]) {
	COMET_clearPulseTemplate(baseAddress);
	COMET_initPulseTemplate(baseAddress, transmitWaveform);	
}

void COMET_configureReceiveEqualizationRAM(int baseAddress, int RAM_Table[256]) {
	int i;
	for(i=0; i < 255; i++) {
		COMET_writeRAM(baseAddress, i, RAM_Table[i]);
	}
}

/*
// Indirect Registers
#define MAX_BUSY_READ 0x5
#define BIT_BUSY 0x80

int COMET_ReadIndirectReg(int offset, unsigned char *value) {
	int i;
	COMET_WriteReg(REG_CHANNEL_INDIRECT_ADDRESS, (offset|0x80));
	for (i = 0; i < MAX_BUSY_READ; i++) {
		*value = COMET_ReadReg(REG_MICRO_ACCESS_STATUS);
		if ((*value&BIT_BUSY) == 0) {
			*value = COMET_ReadReg(REG_CHANNEL_INDIRECT_DATA_BUFFER);
		return SUCCESS;
		}
	}
	return FAILURE;
}

int COMET_WriteIndirectReg(int offset, unsigned char value) {
	unsigned char temp;
	int i;
	COMET_WriteReg(REG_CHANNEL_INDIRECT_DATA_BUFFER, value);
	COMET_WriteReg(REG_CHANNEL_INDIRECT_ADDRESS, (offset&0x7F));
	for (i = 0; i < MAX_BUSY_READ; i++) {
		temp = COMET_ReadReg(REG_MICRO_ACCESS_STATUS);
		if ((temp&BIT_BUSY) == 0) return SUCCESS;
	}
	return FAILURE;
}
*/


